package com.hp.user.service.controller;

import com.hp.user.client.entity.*;
import com.hp.user.client.service.AuditService;
import com.hp.user.client.service.CustomerCompanyService;
import com.hp.user.service.utils.HttpResult;
import com.hp.user.service.utils.IdWorker;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

@Api(tags = "企业/服务商API接口")
@RestController
@RequestMapping("/company")
public class CompanyController {
    @Autowired
    private CustomerCompanyService customerCompanyService;
    @Autowired
    private AuditService auditService;

    /**
     * 添加企业
     *
     * @param company 企业信息
     * @return HttpResult
     */
    @ApiOperation(value = "新增企业", notes = "新增企业", httpMethod = "POST")
    @PostMapping("/addCompany")
    @Transactional
    public HttpResult addCompany(@RequestBody CustomerCompanyVO company, @RequestBody AddressVO address) {
        company.setCustomerId(IdWorker.generactorId().toString());
        boolean flag = customerCompanyService.insert(company);
        if (flag) return HttpResult.success(company.getCustomerId());
        else return HttpResult.fail("添加失败");
    }

    /**
     * 更新企业信息
     *
     * @param company 企业信息
     * @return HttpResult
     */
    @ApiOperation(value = "更新企业信息", notes = "更新企业信息", httpMethod = "POST")
    @PostMapping("/updateCompany")
    public HttpResult updateCompany(@RequestBody CustomerCompanyVO company, @RequestBody AddressVO address) {
        boolean flag = customerCompanyService.update(company);
        if (flag) return HttpResult.success();
        else return HttpResult.fail("更新失败");
    }

    /**
     * 根据企业Id删除企业
     *
     * @param companyId 企业Id
     * @return HttpResult
     */
    @ApiOperation(value = "删除企业", notes = "删除企业", httpMethod = "GET")
    @GetMapping("/deleteCompanyByCompanyId")
    public HttpResult deleteCompanyByCompanyId(@RequestParam String companyId) {
        boolean flag = customerCompanyService.delete(companyId);
        if (flag) return HttpResult.success();
        else return HttpResult.fail("删除失败");
    }

    /**
     * 根据企业Id查询某企业信息
     *
     * @param companyId 企业Id
     * @return HttpResult
     */
    @ApiOperation(value = "根据企业Id查询某企业信息")
    @GetMapping("/queryCompanyByCompanyId")
    public HttpResult queryCompanyByCompanyId(@RequestParam String companyId) {
        CustomerCompanyVO companyVO = customerCompanyService.queryCompanyByCompanyId(companyId);
        if (companyVO != null) return HttpResult.success(companyVO);
        else return HttpResult.fail("查询失败/该企业已被删除");
    }

    /**
     * 分页查询所有企业信息
     *
     * @param current 要查询页数
     * @param size    一页的行数
     * @return HttpResult
     */
    @ApiOperation(value = "分页查询所有企业", notes = "输入的current为要查询的页数，size为一页的行数，返回的total为总行数")
    @GetMapping("/queryAllCompanyByPage")
    public HttpResult queryAllCompanyByPage(@RequestParam Long current, @RequestParam Integer size) {
        CompanyPage companyPage = customerCompanyService.getCompanysPage(current, size);
        if (companyPage != null) return HttpResult.success(companyPage);
        else return HttpResult.fail("查询失败");
    }

    /**
     * 分页查询符合条件的企业信息
     *
     * @param name                企业名称
     * @param serviceProviderName 企业关联服务商名称
     * @param status              审核状态
     * @param current             要查询页数
     * @param size                一页的行数
     * @return HttpResult
     */
    @ApiOperation(value = "根据企业姓名、关联服务商、审核状态分页查询企业", notes = "若查询服务商，则只需要填name，serviceProviderName为空，status为0")
    @GetMapping("/queryCompanysPageByNameAndServiceProviderAndStatus")
    public HttpResult queryCompanysPageByNameAndServiceProviderAndStatus(String name, String serviceProviderName, @RequestParam Integer status, @RequestParam Long current, @RequestParam Integer size) {
        CompanyPage companyPage = customerCompanyService.queryCompanysPageByNameAndServiceAndStatus(name, serviceProviderName, status, current, size);
        if (companyPage != null) return HttpResult.success(companyPage);
        else return HttpResult.fail("查询失败");
    }

    /**
     * 微信端，成为合同用户
     *
     * @param company 企业信息
     * @param willing 是否有意向成为合同用户
     * @return HttpResult
     */
    @ApiOperation(value = "成为合同用户", notes = "添加成功会返回添加的企业Id")
    @PostMapping("/becomeContractUser")
    @Transactional
    public HttpResult becomeContractUser(@RequestBody CustomerCompanyVO company, @RequestBody boolean willing) {
        company.setCustomerId(IdWorker.generactorId().toString());
        AuditVO audit = new AuditVO();
        audit.setAuditId(IdWorker.generactorId().toString());
        audit.setUserId(company.getCustomerId());
        audit.setStatus((short) 1);
        audit.setSource((short) 1);
        // 愿意成为合同用户
        if (willing) audit.setVipLevel((short) 1); // 普通会员
        else audit.setVipLevel((short) 2); // 意向用户
        boolean flag1 = customerCompanyService.insert(company);
        boolean flag2 = auditService.insert(audit);
        if (flag1 && flag2) return HttpResult.success(company.getCustomerId());
        else return HttpResult.fail("添加失败");
    }

    /**
     * 分页查询所有认证企业
     *
     * @param current 要查询页数
     * @param size    一页的行数
     * @return HttpResult
     */
    @ApiOperation(value = "分页查询所有认证企业", notes = "输入的current为要查询的页数，size为一页的行数，返回的total为总行数")
    @GetMapping("/queryAllCompanyCertifyPage")
    public HttpResult queryAllCompanyCertifyPage(@RequestParam Long current, @RequestParam Integer size) {
        CompanyAuditPage companyAuditPage = customerCompanyService.getCompanyAuditsPage(current, size);
        if (companyAuditPage != null) return HttpResult.success(companyAuditPage);
        else return HttpResult.fail("查询失败");
    }

    /**
     * @param name      企业名称
     * @param status    审核状态
     * @param startTime 提交时间的开始间隔
     * @param endTime   提交时间的结束间隔
     * @param current   要查询的页数
     * @param size      一页的行数
     * @return HttpResult
     */
    @ApiOperation(value = "根据企业姓名、审核状态、提交时间分页查询认证企业", notes = "若审核状态为全部，则status填0")
    @GetMapping("/queryCompanysPageByNameAndStatusAndCreateTime")
    public HttpResult queryCompanysPageByNameAndStatusAndCreateTime(String name, @RequestParam Integer status, Date startTime, Date endTime, @RequestParam Long current, @RequestParam Integer size) {
        CompanyAuditPage companyAuditPage = customerCompanyService.queryCompanysPageByNameAndStatusAndCreateTime(name, status, startTime, endTime, current, size);
        if (companyAuditPage != null) return HttpResult.success(companyAuditPage);
        else return HttpResult.fail("查询失败");
    }

//    /**
//     * 查询某企业的所有用户
//     * @param companyId 企业Id
//     */
//    @GetMapping("/queryAllUserOfCompany")
//    public void queryAllUserOfCompany(String companyId){
//
//    }
//
//    /**
//     * 关联某企业与某用户
//     * @param companyId 企业Id
//     * @param userId 用户Id
//     */
//    @GetMapping("/associateCompanyAndUser")
//    public void associateCompanyAndUser(String companyId,String userId){
//
//    }
//
//    /**
//     * 解除某企业与某用户的关联
//     * @param companyId 企业Id
//     * @param userId 用户Id
//     */
//    @GetMapping("/dissociateCompanyAndUser")
//    public void dissociateCompanyAndUser(String companyId,String userId){
//
//    }
}
